import six
# TODO - Add doc strings


#-----------------------------------------------------------------------------#
# Helper Functions
#-----------------------------------------------------------------------------#
def _prompt_for_input(message):
    text_prompt(message, ' ')
    return six.moves.input()


#-----------------------------------------------------------------------------#
# Text Output
#-----------------------------------------------------------------------------#
def _text_out(colour_code, message, end):
    six.print_("\x1b[{}m{}\x1b[0m".format(colour_code, message), end=end)


def text_prompt(message, end="\n"):
    _text_out(33, message, end)


def text_info(message, end="\n"):
    _text_out(37, message, end)


def text_success(message, end="\n"):
    _text_out(32, message, end)


def text_error(message, end="\n"):
    _text_out(31, message, end)


#-----------------------------------------------------------------------------#
# User Input
#-----------------------------------------------------------------------------#
def prompt(message, expected_type='str', default=None):
    while True:
        raw = _prompt_for_input(message)
        if default is not None and not raw:
            raw = default
        if raw and expected_type == 'int':
            try:
                return int(raw)
            except ValueError:
                text_error('Invalid Input (Expected a Number)')
        elif raw:
            return raw
        else:
            text_error('Input required')


def confirm(message, default=False):
    if default:
        default_formatted = "[Yn]"
        default = "y"
    else:
        default_formatted = "[yN]"
        default = "n"
    message = "{} {}:".format(message, default_formatted)
    res = prompt(message, default=default)
    return res.lower().startswith('y')


def choice(message, choices):
    if len(choices) == 1:
        return choices[0][1]
    while True:
        for i, choice in enumerate(choices):
            text_prompt("{} - {}".format(i, choice[0]))
        picked = prompt(message, "int")
        try:
            return choices[picked][1]
        except IndexError:
            text_error("That is not a valid selection")



from __future__ import print_function
from functools import wraps
import sys
# TODO - Add doc strings


#-----------------------------------------------------------------------------#
def catch_exit_keys(f):
    """Catch Ctrl-c and Ctrl-d and exit cleanly"""
    @wraps(f)
    def wrapped(*args, **kwargs):
        try:
            return f(*args, **kwargs)
        except KeyboardInterrupt:
            print()
            sys.exit(0)
        except EOFError:
            print()
            sys.exit(0)
    return wrapped



from distutils.core import setup

import navigator


setup(
    name="Navigator",
    version=navigator.__version__,
    author="Thomas O'Donnell",
    author_email="",
    url="https://github.com/Andytom/navigator",
    packages=["navigator"],
    long_description=open("README.rst").read(),
    install_requires=[
        "six == 1.3.0"
    ]
)



from functools import wraps
import sys

from . import ui, decorators
# TODO - Add doc strings
# TODO - Add better comments


#-----------------------------------------------------------------------------#
# Helper Functions
#-----------------------------------------------------------------------------#
def do_nothing():
    pass


#-----------------------------------------------------------------------------#
# Main Classes
#-----------------------------------------------------------------------------#
class Navigator(object):
    def __init__(self, message="What do you want to do?", intro=None,
                 done_name='quit', no_confirm=True):
        self.actors = {}
        self.message = message
        self.intro = intro
        self.completed = Actor(done_name, sys.exit)
        self.no_confirm = no_confirm

    def _add_actor(self, actor):
        if actor.name in self.actors:
            raise NameError("Name '{}' is already assigned".format(actor.name))
        self.actors[actor.name] = actor

    def route(self, name, blurb=""):
        """Decorator for registering functions"""
        def inner(f):
            actor = Actor(name, f, blurb)
            self._add_actor(actor)

            @wraps(f)
            def wrapped(*args, **kwargs):
                return f(*args, **kwargs)
            return wrapped
        return inner

    def register_assistant(self, assistant):
        self._add_actor(assistant)

    def __repr__(self):
        return "<Navigator - {}>".format(self.intro)

    def display_info(self):
        if self.intro is not None:
            ui.text_info(self.intro)

    @decorators.catch_exit_keys
    def _do_run(self):
        choices = [(self.completed.label, self.completed)]
        for key in iter(self.actors):
            actor = self.actors[key]
            choices.append((actor.label, actor))
        picked = ui.choice(self.message, choices)
        if self.no_confirm or ui.confirm("Run {}?".format(picked.name), True):
            picked.run()

    def run(self):
        self.display_info()
        while True:
            self._do_run()


class Assistant(Navigator):
    def __init__(self, name, blurb, message="What do you want to do?",
                 done_name='back', no_confirm=True):
        super(Assistant, self).__init__(message=message, no_confirm=no_confirm)
        self.blurb = blurb
        self.name = name
        self.label = "{} - {}".format(name, blurb)
        self.completed = Actor(done_name, do_nothing)

    def __repr__(self):
        return "<Assistant {}>".format(self.label)

    def display_info(self):
        ui.text_info(self.label)

    def run(self):
        self._do_run()


class Actor(object):
    def __init__(self, name, func, blurb=""):
        self.name = name
        self.blurb = blurb
        self.func = func
        if blurb:
            self.label = "{} - {}".format(name, blurb)
        else:
            self.label = name

    def __repr__(self):
        return "<Actor {}>".format(self.label)

    def run(self):
        return self.func()
